function gsa_ = gsa_sdp(y, x, tvp, nvr, opts, nvrT, ifig, fname, pnames, yi_anal)

% copyright Marco Ratto 2006
%
% function gsa_ =
% gsa_sdp(y, x, tvp, nvr, opts, nvrT, ifig, fname, pnames, yi_anal)
%         1  2  3    4    5     6     7     8      9       10
% INPUTS
% y, the output
% x, the inputs
% tvp: tipe of estimation
%      0 = load previous estimation
%      1 = RW for all parameters
%      2 = IRW for all parameters
%      -1 = RW for all parameters, fixed nvr's (no estimation)
%      -2 = IRW for all parameters, fixed nvr's (no estimation)
%      an array of length(x) specifies the process for each parameter
% nvr  initial nvr value for estimation or fixed nvr is no estimation
% opts = [iter tcrit nest nsax tstcrit irank ialias RegressorType]
%    iter: max number of backfitting iterations (40-default)
%    tcrit: convergence criterion of backfitting (0.001)
%    nest: number of backfitting iteration with ML estimation (2-default)
%    nsax: number of samples actually used for optimization and for the basis
%          dimension: [ min(256, length(y)) is default ]
%    tstcrit: critical value for variable selection using t-stat of Si (1-default)
%    irank = 1 to rank transform (1-default)
%          = 0 not transform: NEEDS uniform inputs [0 1]!!!!
%    ialias: to be used with LPTAU samples < 1024 (0-default)
%    RegressorType: 1 for unity regressor (default)
%                   0 for X regressor
%    ord  ord=1: parametric linear term (default)
%         ord=2: parametric linear and saddle terms!
% nvrT maximum nvr allowable
% ifig: 1 plot figures, 0 no plots
% fname, file name to save the analysis
% pnames: names of the params
% yi_anal, analytical value of the first order HDMR terms (optional)
%
% OUTPUT
%   gsa_.f0  : mean output
%   gsa_.f   : main effects
%   gsa_.fs  : sorted main effects
%   gsa_.fses: sorted st. err. of main effects
%   gsa_.nvr : estimated nvr's
%   gsa_.r2  : r2 of the sum of main effects 
%   gsa_.si  : sensitivity indices
%   gsa_.si_std: standard error of si (Doksum-Samarov)
%   gsa_.stat: t-stat of si
%   gsa_.x0  : original inputs 
%   gsa_.xx  : rank transformed inputs
%   gsa_.y   : original output
% 
% f, function estimates
% fs, sorted function estimates
% fses, sorted standard error of function estimates
% si, sensitivity indices
% si_std, st. error of sensitivity indices
% stat, tstat for significance of si
% xx, transformed inputs used (rank-transformed)
% x0, original inputs
% y, output

if nargin==0,
  disp(' ')
  disp('GSA_SDP: SDP estimation of first order HDMR terms')
  disp(' ')
  disp('gsa_ = gsa_sdp(y, x, tvp, nvr, opts, nvrT, ifig, fname, pnames, yi_anal)')
  disp('               1  2  3    4    5     6     7     8      9       10 ')
  disp(' ')
  return
end

% RegressorType=1;  % CONST regressor
% % RegressorType=0; % X regressor

[kn, np]=size(x);
if nargin<3 | isempty(tvp),
  tvp=-2;
  TVP=ones(1,np).*(abs(tvp)-1);
elseif length(tvp)==1,
  if tvp>0
    TVP=ones(1,np).*(tvp-1);
  else
    TVP=ones(1,np).*(abs(tvp)-1);
  end
else
  TVP=tvp-1;    
end

if nargin<4 | isempty(nvr)
  nvr0=[];
  bigT=(kn)/3;
%   bigT=(kn-length(find(isnan(y))))/3;
%   nvr1 = ( 2*sin( 0.5*(2*pi/(kn/2)) ) )^2;
%   nvr2 = ( 2*sin( 0.5*(2*pi/(kn/2)) ) )^4;
  nvr1 =  2*(1-cos( 2*pi/bigT ) );
  nvr2 = (2*(1-cos( 2*pi/bigT ) ))^2;
else
  nvr2=nvr;
  nvr1=nvr;
  nvr0=nvr;
end
if length(nvr1)==1,
  nvr1 = nvr1(ones(np,1));  
end  
if length(nvr0)==1,
  nvr0 = nvr0(ones(np,1));  
end  
if length(nvr2)==1,
  nvr2 = nvr2(ones(np,1));  
end  

if nargin<5 | isempty(opts)
    opts = [40 0.001 2 min(kn, 256) 1 1 0 1 1];
end
opts0=[40 0.001 2 min(kn, 256) 1 1 0 1 1];
if length(opts)<9;
    opts=[opts opts0(length(opts)+1:9)];
end

opts(find(opts<0))=opts0(find(opts<0));

iter=opts(1);
tcrit=opts(2);
nest=opts(3);
nsax=opts(4);
tstcrit=opts(5);
irank=opts(6);
ialias=opts(7);
RegressorType=opts(8);  % 1 = CONST regressor; 0= X regressor
ord=opts(9);


if nargin<6 | isempty(nvrT),
  nvrT=16;
end
if nargin<7 | isempty(ifig),
  ifig=0;
end
if nargin<8 | isempty(fname),
  fname='gsa_sdp';
end

if nargin<10 | isempty(yi_anal),
  yi_anal=[];
end

if tvp~=0,
yin=y;
iok=find(~isnan(y));
inok=find(isnan(y));
[y(iok),my,sy]=stand_(y(iok));
x0=x;
% x=stand_(x);
  if irank,
    xx=x*0;
    xx(iok,:)=trank(x(iok,:))-1/(2*length(iok));
    if ~isempty(inok),
    for j=1:np,
      if max(x(inok,j))>max(x(iok,j))
        xout1=max(x(inok,j));
        xxout1=1;
      else
        xout1=[];
        xxout1=[];
      end
      if min(x(inok,j))<min(x(iok,j))
        xout0=min(x(inok,j));
        xxout0=0;
      else
        xout0=[];
        xxout0=[];
      end
      
      xx(inok,j)=interp1([x(iok,j); xout0; xout1],[xx(iok,j); xxout0; xxout1],x(inok,j));
    end
    end
    x=xx;
    if RegressorType==0,
      x=x+1;
    end
  else
%     if max(max(x))>1 | min(min(x))<0,
%       error('irank=0: Inputs MUST be in the unit hypercube!')
%     end
    xx=x;

  end
  %nvr2 = ( 2*sin( 0.5*(2*pi/(3*(kn/4))) ) )^4;
  
  
  if RegressorType==1,
    xr=ones(size(x));
  else
    xr=x;
  end

  if ialias,
  opts=[1 tcrit -1 -1 -1 -1]; %[iter con meth sm ALG plotopt]
  for jj=1:length(ialias),
    j=ialias(jj);
    
    lastwarn('')
    if tvp<0
      if tvp==-1
        [fit,fitse,par(:,j),parse(:,j),zs(:,j),pars(:,j),parses(:,j),rsq(j),nvre(j)]= ...
          sdr(y,xr(:,j),x(:,j), 0, nvr1(j),opts);
      else
        [fit,fitse,par(:,j),parse(:,j),zs(:,j),pars(:,j),parses(:,j),rsq(j),nvre(j)]= ...
          sdr(y,xr(:,j),x(:,j), 1, nvr2(j),opts);
      end
    else
      if isempty(nvr0),
        nvr00=[];
      else
        nvr00=nvr0(j);
      end
      [fit,fitse,par(:,j),parse(:,j),zs(:,j),pars(:,j),parses(:,j),rsq(j),nvre(j)]= ...
        sdr(y,xr(:,j),x(:,j), TVP(j), -1,opts,[],[],nvr00,0);
      if nvre(j)>nvrT,
        nvre(j)=nvrT;
        [fit,fitse,par(:,j),parse(:,j),zs(:,j),pars(:,j),parses(:,j),rsq(j),nvre(j)]= ...
          sdr(y,xr(:,j),x(:,j), TVP(j), nvre(j),opts);
      end
        
    end
    
    if RegressorType==1,
      zs=xr;
    end 
    dwarn=lastwarn;
    if length(dwarn)>0,
      nvre(j)=0;
    end  
    m0=mean(zs(:,j).*pars(:,j));
    %siu(1, j) = var(zs(:,j).*pars(:,j));
    si(1, j) = var(xr(iok,j).*par(iok,j)-m0);
    stat(1, j) = mean(abs([(par(iok,j)-mean(par(:,j)))./parse(iok,j)])) ;         
    rT(1, j) = rsq(j);
    f(:,j) =(xr(:,j).*par(:,j)-m0).*sy;
    fs(:,j)=(zs(:,j).*pars(:,j)-m0).*sy; 
    fses(:,j)=zs(:,j).*parses(:,j).*sy;
    % Doksum-Samarov estimate of the error
    u = (y(iok)-f(iok,j)./sy)/sqrt(1-si(j));
    sie(1, j)=(1-si(j))*std(y(iok).^2-u.^2)/sqrt(length(y));
    tst(1,j)=si(j)/max(eps,sie(j));
    
    lmod(j) = {'RW'};
    if TVP(j), lmod(j)={'IRW'}; end,

  end
  end
  
  imult=[1:np];
  imult=imult(find(imult~=ialias));
  opts=[iter tcrit -1 -1 -1 0 tstcrit]; %[iter con meth sm ALG plotopt tstcrit]
  y0=y;
  if tvp<0,
    if tvp==-1
      nvrP=nvr1;
    else
      nvrP=nvr2;
    end
    if ialias,
      y0 = y0 - sum(f(:,ialias)./sy,2);
    end
  else
    nvrP=ones(1,np).*(-nest);
    if ialias,
      nvrP(ialias)=nvre(ialias);  % already estimated !!
      y0 = y0 - sum(f(:,ialias)./sy,2);
    end
  end

  if tvp<0
    [fit2,fitse2,par2,parse2,zs2,pars2,parses2,rsq2,nvre2,si2]= ...
      sdrnew(y0,xr(:,imult),x(:,imult),abs(tvp)-1,nvrP(imult),opts,[],[],[],[],ord);
%       sdrspl(y0,xr(:,imult),x(:,imult),abs(tvp)-1,nvrP(imult),opts);
  else
    if isempty(nvr0),
      nvr00=nvr0;
    else
      nvr00=nvr0(imult);
    end
    [fit2,fitse2,par2,parse2,zs2,pars2,parses2,rsq2,nvre2,si2]= ...
      sdrnew(y0,xr(:,imult),x(:,imult),TVP(imult),nvrP(imult),opts,[],[],nvr00,0,ord);
%       sdrspl(y0,xr(:,imult),x(:,imult),TVP(imult),nvrP(imult),opts,[],[],nvr00,0);
  end
  if any(nvre2>nvrT),
    ik=find(nvre2>nvrT);
    nvre2(ik)=ones(1,length(ik)).*nvrT;
    [fit2,fitse2,par2,parse2,zs2,pars2,parses2,rsq2,nvre2]= ...
      sdrnew(y0,xr(:,imult),x(:,imult),TVP(imult),nvre2, opts);
  end
  par(:,imult)=par2;
  parse(:,imult)=parse2;
  zs(:,imult)=zs2;
  pars(:,imult)=pars2;
  parses(:,imult)=parses2;
  nvre(imult)=nvre2;
  si(imult)=si2(1,:);
  sie(imult)=si2(2,:);
  tst(imult)=si2(3,:);
  if RegressorType==1,
    zs=xr;
  end
  
  
  for j=1:np,
    m0=mean(zs(:,j).*pars(:,j));
%     if ismember(j,imult),
%       %si(1, j) = var(zs(:,j).*pars(:,j));
%       si(1, j) = var(xr(iok,j).*par(iok,j));
%       stat(1, j) = max(abs([(par(iok,j)-mean(par(:,j)))./parse(iok,j)])) ;         
%     end
    f(:,j)=(xr(:,j).*par(:,j)-m0).*sy;
    fs(:,j)=(zs(:,j).*pars(:,j)-m0).*sy;
    fses(:,j)=zs(:,j).*parses(:,j).*sy;
    % Doksum-Samarov estimate of the error
%     u = (y(iok)-f(iok,j)./sy)/sqrt(1-si(j));
%     if ismember(j,imult),
%       sie(1,j)=(1-si(j))*std(y(iok).^2-u.^2)/sqrt(length(u));
%       tst(1,j)=si(j)/sie(j);
%     end

  end
  ino= find(tst<=tstcrit);
  iyes= find(tst>tstcrit);
%   f(:,ino)=f(:,ino)*0;
%   fs(:,ino)=fs(:,ino)*0;
  rsq = 1-cov(y(iok)-sum(f(iok,iyes),2)./sy)/cov(y(iok));
  chk=zeros(np,1);
  chk(iyes)=1;
  if kn<2500,
    [yt, outk] = cubic_spl_k(xx(:,logical(chk)), yin, 1./nvre(logical(chk))./kn^4);
    outk.chk=chk;
    outk.nvr=nvre;
  else
    outk=[];
    yt=sum(f(iok,iyes),2);
  end
  gsa_.fit=yt;
  gsa_.f0=my;
  gsa_.f=f;
  gsa_.fs=fs;
  gsa_.fses=fses;
  gsa_.nvr=nvre;
  gsa_.r2=rsq;
  gsa_.si=si;
  gsa_.si_std=sie;
  gsa_.stat=tst;
  gsa_.x0=x0;
  gsa_.xx=xx;
  gsa_.y=yin;
  gsa_.chk=chk;
  gsa_.out=outk;
%   save(fname,'gsa_')
else
%   load(fname)
  gsa_=y;
%   xx=gsa_.xx;
%   yin=gsa_.y;
%   [xcum, paracdf] = cumdens(x,pshape,p1,p2);
%   pcoef = ChRegressGSA(xcum,8,1,gsa_..f);
%   for ij=1:np,
%     scoef{ij}=['[',num2str(pcoef(ij,:),'%20.16e '),']'];
%     gg{ij}=inline(['polyval(',scoef{ij},',',char(paracdf{ij}),')'],'X');
%   end
%   gsa_.univariate.f0=my;
%   gsa_.f0=my;
%   gsa_.gpred=gg;
%   save(fname,'gsa_')
end  

if ifig,
  if nargin<9 | isempty(pnames),
    pnames=['X_{',num2str(1),'}'];
    for j=2:np,
      pnames=str2mat(pnames,['X_{',num2str(j),'}']);
    end
    
  end
  if nargin<10,
    yi_anal=[];
  end
  hfig = gsa_sdp_plot(gsa_,fname,pnames,[1:np], yi_anal);
  close(hfig);
%   nplo = ceil(sqrt(np));
%   if nplo*(nplo-2)>=np,
%     nplo2=nplo-2;
%   elseif nplo*(nplo-1)>=np,
%     nplo2=nplo-1;    
%   elseif (nplo+1)*(nplo-1)>=np,
%     nplo2=nplo-1;    
%     nplo=nplo+1;    
%   else
%     nplo2=nplo;
%   end
%   
%   if nargin<8 | isempty(pnames),
%     pnames=['X_{',num2str(1),'}'];
%     for j=2:np,
%       pnames=str2mat(pnames,['X_{',num2str(j),'}']);
%     end
%     
%   end
%   
%   figure('name',['Nrun=',num2str(kn),', ',fname,', SDP']),
%   f=gsa_.f;
%   fs=gsa_.fs;
%   fses=gsa_.fses;
%   stat=gsa_.stat;
%   v_di_e(1,:)=gsa_.si;
%   for j=1:np,
%     %subplot(3,4,j)
%     subplot(nplo2,nplo,j)
%     xp=sort(gsa_.x0(:,j));
%     %xp=zs(:,jj)-1;
%     if exist('yi_anal') & ~isempty(yi_anal),
%       %plot(xp, yi_anal(:,j)./sy,'r:'), hold on, 
%       plot(xp, yi_anal(:,j),'r:'), hold on, 
%     end
%     plot(xp, fs(:,j),'k', xp, 2.*fses(:,j),':k', xp, -2.*fses(:,j),':k')
%     set(gca,'xlim',[min(xp) max(xp)])        
%     lmod(j) = {'I1'};
%     if TVP(j), lmod(j)={'I2'}; end,
%     title([deblank(pnames(j,:)),', Si=',num2str(v_di_e(1,j),'%5.2f')],'fontsize',9,'interpreter','tex')
%   end
%   saveas(gcf,[fname,'_SE'])
%   eval(['print -depsc2 ' fname '_SE']);
%   eval(['print -dpdf ' fname '_SE']);
%   close(gcf)
  
%   figure('name',['Nrun=',num2str(kn),', ',fname,', SDP univariate']),    
%   f=gsa_.univariate.f;
%   fs=gsa_.univariate.fs;
%   fses=gsa_.univariate.fses;
%   vi(1,:)=gsa_.univariate.si;
%   vi(2,:)=gsa_.univariate.si_std;
%   for j=1:np,
%     subplot(nplo2,nplo,j)
%     xp=sort(x0(:,j));
%     if exist('yi_anal') & ~isempty(yi_anal),
%       plot(xp, yi_anal(:,j),'r'), hold on, 
%     end
%     plot(xp, [fs(:,j) 3.09.*fses(:,j) -3.09.*fses(:,j)])
%     set(gca,'xlim',[min(xp) max(xp)])        
%     lmod(j) = {'I1'};
%     if TVP(j), lmod(j)={'I2'}; end,
%     title([deblank(pnames(j,:)),', ',lmod{j},', tstat=',num2str(vi(2,j),'%5.2f'), ...
%         ', S=',num2str(vi(1,j),'%5.2f')])
%   end
%   %saveas(gcf,[fname,'_SE_univ'])
%   close(gcf)
end
%end

