function [gsa1, gsa2]=ss_anova(y,x,ord,icomp,TVP,nvr1,nvr2,opts1,opts2,fname);
% 
% copyright Marco Ratto 2008, Joint Research Centre (JRC)
%
%[gsa1, gsa2]=ss_anova(y, x, ord, icomp, TVP, nvr1, nvr2, opts1, opts2, fname);
%                      1  2  3    4      5    6     7     8      9      10
%
% Smoothing spline ANOVA using RECURSIVE algorithms
%
% INPUT
% y, the output
% x, the inputs: preferably provide uniform samples in UNIT hypercube,
%    i.e. if any generic distribution is used for x's, provide the cumulative values for x
%    by default, otherwise, x will be rank-transformed (NOT re-scaled into [0 1]).
%   
% ord: order of the ANOVA 1 or 2 (default=1)
% icomp:
%        0 = use previously estimated nvr's
%        1 = begin new estimation (default)
%        2 = begin new estimation with preliminary tcubic optimization with
%            single lambda
% TVP: TVP's for first and second order terms [TVP1 TVP2]. 2= IRW; 1=RW
% nvr1  initial nvr's for first order effects if estimation or fixed nvr if no estimation
%      use negative value for terms to be left out  from ANOVA
% nvr2  initial nvr's for second order effects if estimation or fixed nvr if no estimation
%      use negative value for terms to be left out  from ANOVA
% opts 1= [iter tcrit nest nsax tstcrit irank ialias RegressorType]
%    iter: max number of backfitting iterations (40-default)
%    tcrit: convergence criterion of backfitting (0.001)
%    nest: number of backfitting iteration with ML estimation (2-default)
%    nsax: number of samples actually used for optimization and for the basis
%          dimension: [ min(256, length(y)) is default ]
%    tstcrit: critical value for variable selection using t-stat of Si (1-default)
%    irank = 1 to rank transform (1-default)
%          = 0 not transform: NEEDS uniform inputs [0 1]!!!!
%    ialias: to be used with LPTAU samples < 1024 (0-default)
%    RegressorType: 1 for unity regressor (default)
%                   0 for X regressor
% opts2: estimation options [iter tcrit nest nsax tstcrit irank ialias lper bkp_opt], use -1 for defaults
%                           
%       iter: number of backfitting iterations on interactions (2-default)
%       tcrit: convergence criterion of backfitting (0.001)
%       nest: number of backfitting loops with ML estimation (2)
%       nsax: number of samples actually used for optimization and for the basis
%          dimension: [ min(256, length(y)) is default ]
%       tscrit: critical value for tstat for regressor selection
%       irank: = 1 to rank transform x data (default)
%              = 0 do transformation: NEEDS uniform inputs [0 1]!!!!
%       ialias: specify if some main effects have NOT to be taken out from y 
%              (for lptau samples < 1024 runs, column 14 can have aliasing )
%       lper: number of sample in the semi-period for sort2 (16)
%       bkp_opt: keep (=1) or delete (=0) intermediate bkp files saved at
%              each backfitting iteration (0)
% fname, file name to save the analysis
%
% OUTPUT
%   Main effects:
%   gsa1.fit : fitted output
%   gsa1.f0  : mean output (zero order term)
%   gsa1.f   : main effects
%   gsa1.fs  : sorted main effects
%   gsa1.fses: sorted st. err. of main effects
%   gsa1.nvr : estimated nvr's
%   gsa1.r2  : r2 of the sum of main effects 
%   gsa1.si  : sensitivity indices
%   gsa1.si_std: standard error of si (Doksum-Samarov)
%   gsa1.stat: t-stat of si
%   gsa1.x0  : original inputs 
%   gsa1.xx  : actual inputs for reproducing kernel
%   gsa1.y   : original output
%   gsa1.chk: logical array of included (=1) or excluded (=0) 1st order terms
%   gsa1.out : information on the reproducing kernel of the 1st order model
%
%   Second order effects:
%   gsa2.fit : fitted output with 2nd order model
%   gsa2.f   : second order functional effects
%   gsa2.fse : S.E. of second order effects
%   gsa2.nvr : estimated nvr's
%   gsa2.s22 : square matirx of Sij's
%   gsa2.s2  : stacked arrayif Sij's
%   gsa2.s2_std: S.E. of Sij's
%   gsa2.stat: t-stat of Sij's
%   gsa2.chk: logical array of included (=1) or excluded (=0) 2nd order terms
%   gsa2.out: structure with all information about the Reproducing Kernel.
%
% The internal SDR routines used in this toolbox
% have been developed by Lancaster University in conjunction with the JRC.
% The SDR routine is a special purpose version of the 'full' SDP algorithm,
% developed by Prof. Peter Young and co-workers at Lancaster University. 
%




if nargin==0,
  disp(' ')
  disp('[gsa1, gsa2]=ss_anova(y, x, ord, icomp, nvr1, nvr2, opts1, opts2, fname);')
  disp('                      1  2  3    4      5     6     7      8      9')
  disp(' ')
  return
end

if nargin<2
  error('Minimum 2 inputs are required!')
end

if nargin<3 | isempty(ord),
  ord=1;
end
if nargin<4 | isempty(icomp),
  icomp=1;
end
if nargin<5 | isempty(TVP),
  TVP2=1;
  TVP=2;
else
  TVP2=TVP(end);
  TVP=TVP(1);
end
if nargin<6,
  nvr1=[];
end
if nargin<7,
  nvr2=[];
end
if nargin<8 | isempty(opts1)
  if min(min(x))<0 | max(max(x))>1,
    opts1=[-1 -1 -1 -1 1 1]; %[iter tcrit nest nsax tstcrit irank ialias]
  else 
    opts1=[-1 -1 -1 -1 1 0]; %[iter tcrit nest nsax tstcrit irank ialias]
  end
  opts2 = opts1; % [iter tcrit nest tstcrit irank ialias lper bkp_opt]
end
if nargin<9 | isempty(opts2),
  opts2 = opts1; % [iter tcrit nest tstcrit irank ialias lper bkp_opt]
end  

if nargin<10 | isempty(fname),
  fname='ss_anova';
end
fname2=[fname,'_2nd'];

if isstruct(icomp),
  out=icomp;
  icomp=2;
end

if icomp==0,
  TVP=-TVP;
  TVP2=-TVP2;
  opts2(3)=0;
end
    [kn, np]=size(x);

if isempty(nvr1),
  chk1=[];
else
  chk1=ones(np ,1);
  chk1((nvr1<0))=0;
  nvr1((nvr1<0))=0;
end

if icomp==0 & ord==2, 
  gsa1.fit=[];
  gsa1.f0=[];
  gsa1.f=[];
  gsa1.fs=[];
  gsa1.fses=[];
  gsa1.nvr=nvr1;
  gsa1.r2=[];
  gsa1.si=[];
  gsa1.si_std=[];
  gsa1.stat=[];
  gsa1.x0=[];
  gsa1.xx=[];
  gsa1.y=[];
  gsa1.chk=chk1;
else
  tic,
  yt=y*0;
  if icomp==2,
%     out = acosso_fn(x,y,ord,1,0);
%     [yt, lam, out, gsa0] = tcubic_opt(x,y,ord,1);
    if ord<2,
      yt=yt*0;
    else
      yt=sum(out.f(:,np+1:end),2);
      nvr2=1/out.lam/kn^3; % lam in acosso e' lam*n !!!!!
      nvr2=ones(1,nchoosek(np,2))*nvr2;
      chk2=sign(out.theta(np+1:end));
%       yt=sum(gsa0.f2,2);
%       nvr2=1/lam/kn^4;
%       nvr2=ones(1,nchoosek(np,2))*nvr2;
%       chk2=gsa0.chk2;
    end
    nvr1=1/out.lam/kn^3; % lam in acosso e' lam*n !!!!!
    chk1=sign(out.theta(1:np));
%     nvr1=1/lam/kn^4;
%     chk1=gsa0.chk1;
  end    
  gsa1 = gsa_sdp(y-yt, x, TVP, nvr1, opts1, [], [], fname);
  if icomp==2,
    gsa1.theta=out.theta; gsa1.M=out.M;
  end

  toc,
end,
if ord==2,
  if nargout==2,
  % 2nd order interactions
    if isempty(nvr2),
      chk2=[];
    else
      if length(nvr2)==1,
        if nvr2>0,
          chk2=1;
        else
          chk2=0;
          nvr2=0;
        end
      elseif icomp~=2,
      chk2=ones(nchoosek(np,2) ,1);
      chk2((nvr2(1,:)<0))=0;
      if  size(nvr2,1)>1,
        nvr2(2,(nvr2(1,:)<0))=0;
        nvr2(3,(nvr2(1,:)<0))=0;
      end
      
      nvr2(1,(nvr2(1,:)<0))=0;
      end
    end
    if icomp==2,
      [gsa2, gsa1] = gsa_sdp_spl_2nd(y, x, {gsa1, out}, TVP2, [], opts2, nvr2, chk2, [],fname2);
    else
      [gsa2, gsa1] = gsa_sdp_spl_2nd(y, x, gsa1, TVP2, [], opts2, nvr2, chk2, [],fname2);
    end
      
    i0=0;
    istart=zeros(1,np);
    for j=1:np,
      if  j==1,
        iblo=[];
      else
        istart(1:j-2)=istart(1:j-2)+1;
        istart(j-1)=istart(j-1)+jblo(j-1,1);
      end
      jblo(j,:)=[(i0+1),(i0+np-j)];        
      iblo=[istart(1:j-1) [(i0+1):(i0+np-j)]];
      i0=iblo(end);
      gsa2.sT(j)=gsa1.si(j)+sum(gsa2.s2(iblo));      
    end
  else
    error('You need 2 output arguments with ORD=2!')
  end
end